
(*
	----------------------------------------------

	  acmEnum.dpr
	  Voice Communicator II
	  Audio Tools - simple ACM enumeration application

	----------------------------------------------
	  This source code cannot be used without
	  proper permission granted to you as a private
	  person or an entity by the Lake of Soft, Ltd

	  Visit http://lakeofsoft.com/ for details.

	  Copyright (c) 2001, 2002 Lake of Soft, Ltd
		     All rights reserved
	----------------------------------------------

	  created by:
		Lake, Sep 2001

	  modified by:
		Lake, 26 Jan 2002
		Lake, 04 Feb 2002
		Lake, 17 Mar 2002

	----------------------------------------------
*)

{$I unaDef.inc}

{$APPTYPE CONSOLE}

program
  acmEnum;

uses
  Windows, MMSystem,
  unaTypes, unaUtils, unaMsAcmAPI;

// --  --
function htmlFix(const value: string): string;
var
  i: int;
begin
  i := 1;
  result := '';
  //
  while (i <= length(value)) do begin
    //
    case (value[i]) of

      '&':
	result := result + '&amp;';

      '':
	result := result + '(c)';

      '':
	result := result + '(r)';

      #160:
	result := result + ' ';

      else
	result := result + value[i];

    end;
    //
    inc(i);
  end;
end;

// -- --
function WriteWave(const wave: WAVEFORMATEX; size: unsigned; const ident: string = ''): bool;
var
  i: unsigned;
begin
  infoMessage(ident + '<waveformatex tag="' + int2Str(wave.wFormatTag) + '" formatSize="' + int2Str(size) + '">');
  infoMessage(ident + '  <nChannels value="' + int2Str(wave.nChannels) + '" />');
  infoMessage(ident + '  <nSamplesPerSec value="' + int2Str(wave.nSamplesPerSec) + '" />');
  infoMessage(ident + '  <nAvgBytesPerSec value="' + int2Str(wave.nAvgBytesPerSec) + '" />');
  infoMessage(ident + '  <nBlockAlign value="' + int2Str(wave.nBlockAlign) + '" />');
  infoMessage(ident + '  <wBitsPerSample value="' + int2Str(wave.wBitsPerSample) + '" />');
  //
  if (size >= sizeOf(WAVEFORMATEX)) then begin
    infoMessage(ident + '  <cbSize value="' + int2Str(wave.cbSize) + '" />');
    if (wave.cbSize > 0) then begin
      infoMessage(ident + '  <wave_data>');
      i := 0;
      while (i < wave.cbSize) do begin
	infoMessage(adjust(int2Str(byte(pchar(@wave.cbSize)[sizeof(wave.cbSize) + i]), 16), 2, '0') + ' ');
	inc(i);
      end;
      infoMessage('</wave_data>');
    end;
  end;
  //  
  infoMessage(ident + '</waveformatex>');
  result := true;
end;

// -- --
function myFilterTagEnumCB(hadid: HACMDRIVERID; const paftd: ACMFILTERTAGDETAILSA; dwInstance: DWORD; fdwSupport: DWORD): Windows.BOOL; stdcall;
begin
  infoMessage('        <filter tag="' + int2Str(paftd.dwFilterTag) + '"' +
			' info="' + htmlFix(paftd.szFilterTag) + '" />');
  result := true;
end;

// -- --
function myFormatEnumCB(hadid: HACMDRIVERID; const pafd: ACMFORMATDETAILSA; dwInstance: DWORD; fdwSupport: DWORD): Windows.BOOL; stdcall;
begin
  infoMessage('        <format tag="' + int2Str(pafd.dwFormatTag) + '"' +
			' index="' + int2Str(pafd.dwFormatIndex) + '"' +
			' info="' + htmlFix(pafd.szFormat) + '" >');
  WriteWave(pafd.pwfx^, pafd.cbwfx, '          ');
  infoMessage('        </format>');
  result := true;
end;

// -- --
function myFormatTagEnumCB(hadid: HACMDRIVERID; const paftd: ACMFORMATTAGDETAILSA; had: unsigned; fdwSupport: unsigned): Windows.BOOL; stdcall;
var
  afd: ACMFORMATDETAILS;
  pwfx: pWAVEFORMATEX;
  size: unsigned;
begin
  infoMessage('    <format_tag tag="' + int2Str(paftd.dwFormatTag) + '"' +
		       ' index="' + int2Str(paftd.dwFormatTagIndex) + '"' +
		       '  info="' + htmlFix(paftd.szFormatTag) + '" >');
  infoMessage('      <format_size>' + int2Str(paftd.cbFormatSize) +
	       '</format_size>');
  infoMessage('      <num_formats>' + int2Str(paftd.cStandardFormats) +
	       '</num_formats>');
  infoMessage('      <formats>');

  size := getMaxWaveFormatSize(hadid);

  getMem(pwfx, size);
  fillChar(pwfx^, size, #0);
  pwfx.wFormatTag := paftd.dwFormatTag;
  if (size >= sizeOf(pwfx^)) then
    pwfx.cbSize := size - sizeof(pwfx^);

  fillChar(afd, sizeof(afd), #0);
  afd.cbStruct := sizeOf(afd);
  afd.dwFormatTag := paftd.dwFormatTag;
  afd.pwfx := pwfx;
  afd.cbwfx := size;

  acm_FormatEnum(had, afd, @myFormatEnumCB, 0, ACM_FORMATENUMF_WFORMATTAG);
  freeMem(pwfx);

  infoMessage('      </formats>');
  infoMessage('    </format_tag>');
  result := true;
end;

// --  --
procedure displayVersion(const tagName: string; value: unsigned);
begin
  infoMessage('  <' + tagName + '>' + int2Str((value and $FF000000) shr 24) + '.' +
			      int2Str((value and $00FF0000) shr 16) + ', build ' +
			      int2Str((value and $0000FFFF)) +
	   '</' + tagName + '>');
end;

// -- myDriverEnumCB --
function myDriverEnumCB(hadid: HACMDRIVERID; instance: DWORD; support: DWORD): Windows.BOOL; stdcall;
var
  details: ACMDRIVERDETAILS;
  paftd: ACMFORMATTAGDETAILS;
  pafild: ACMFILTERTAGDETAILS;
  had: HACMDRIVER;
begin
  fillChar(details, sizeOf(details), #0);
  details.cbStruct := sizeOf(details);

  acm_DriverDetails(hadid, details, 0);

  infoMessage('<driver '{id="0x' + int2Str(hadid, 16) + '"'} +
		//' details_size="' + int2Str(details.cbStruct) + '"' +
		' info="' + htmlFix(details.szShortName) + '"' +
		' mid="' + int2Str(details.wMid) + '"' +
		' midInfo="' + mid2Str(details.wMid) + '"' +
		' pid="' + int2Str(details.wPid) + '" >');

  infoMessage('  <type>' + choice(ACMDRIVERDETAILS_FCCTYPE_AUDIOCODEC = details.fccType, 'codec', 'unknown') +
	   '</type>');

  infoMessage('  <subtype>' + int2Str(details.fccComp) +
	   '</subtype>');

  displayVersion('expected_acm_version', details.vdwACM);
  displayVersion('driver_version', details.vdwDriver);

  infoMessage('  <support>' + choice(0 <> (details.fdwSupport and ACMDRIVERDETAILS_SUPPORTF_ASYNC),     'ASYNC (supports asynchronous conversions)', '') +
			      choice(0 <> (details.fdwSupport and ACMDRIVERDETAILS_SUPPORTF_CODEC),     ' CODEC (supports conversion between two different format tags)', '') +
			      choice(0 <> (details.fdwSupport and ACMDRIVERDETAILS_SUPPORTF_CONVERTER), ' CONVERTER (supports conversion between two different formats of the same format tag)', '') +
			      choice(0 <> (details.fdwSupport and ACMDRIVERDETAILS_SUPPORTF_DISABLED),  ' DISABLED (has been disabled)', '') +
			      choice(0 <> (details.fdwSupport and ACMDRIVERDETAILS_SUPPORTF_FILTER),    ' FILTER (supports a modification of the data without changing any of the format attributes)', '') +
			      choice(0 <> (details.fdwSupport and ACMDRIVERDETAILS_SUPPORTF_HARDWARE),  ' HARDWARE (supports hardware input, output, or both through a waveform-audio device)', '') +
			      choice(0 <> (details.fdwSupport and ACMDRIVERDETAILS_SUPPORTF_LOCAL),     ' LOCAL (has been installed locally with respect to the current task)', '') +
	      '</support>');

  infoMessage('  <num_tags>' + int2Str(details.cFormatTags) +
	   '</num_tags>');

  infoMessage('  <num_filters>' + int2Str(details.cFilterTags) +
	   '</num_filters>');

  infoMessage('  <hicon>0x' + int2Str(details.hicon, 16) +
	   '</hicon>');

  infoMessage('  <description>');
  infoMessage('    <short_name>' + htmlFix(details.szShortName) + '</short_name>');
  infoMessage('    <long_name>' + htmlFix(details.szLongName) + '</long_name>');
  infoMessage('    <copyright>' + htmlFix(details.szCopyright) + '</copyright>');
  infoMessage('    <licensing>' + htmlFix(details.szLicensing) + '</licensing>');
  infoMessage('    <features>' + htmlFix(details.szFeatures) + '</features>');
  infoMessage(' </description>');

  if (0 = acm_DriverOpen(had, hadid, 0)) then begin
    infoMessage('');
    infoMessage('   <format_tags>');
    fillChar(paftd, sizeof(paftd), #0);
    paftd.cbStruct := sizeof(paftd);
    acm_FormatTagEnum(had, paftd, @myFormatTagEnumCB, had, 0);
    infoMessage('  </format_tags>');

    infoMessage('');
    infoMessage('   <filters>');
    fillChar(pafild, sizeof(pafild), #0);
    pafild.cbStruct := sizeof(pafild);
    acm_FilterTagEnum(had, pafild, @myFilterTagEnumCB, had, 0);
    infoMessage('  </filters>');

    acm_DriverClose(had, 0);
  end;

  infoMessage('</driver>');
  result := true;
end;

// --  --
function showAllocMemSize(): bool;
var
  s: string;
begin
  s := '<debug AllocMemSize="' + int2Str(AllocMemSize) + '" />';
  infoMessage(s);
  result := true;
end;

// --  --
procedure init();
begin
  infoMessage('<?xml version="1.0" encoding="UTF-8" standalone="yes"?>');
  infoMessage('<acm>');
  assert(showAllocMemSize());
  infoMessage('<info>' +
		'<author>acmEnum, Version 1.0, Voice Communicator components version 2.0</author>' +
		'<copyright>Copyright (c) 2001-2002 Lake of Soft, Ltd</copyright>' +
		'<legal>Freeware version, visit http://lakeofsoft.com/ for more information</legal>' +
	  '</info>');
  displayVersion('acm_version', acm_GetVersion());	  
end;

// --  --
procedure done();
begin
  assert(showAllocMemSize());
  infoMessage('</acm>');
end;

// -- main --
begin
  init();
  acm_DriverEnum(myDriverEnumCB, 0, 0);
  done();
end.

