
(*
	----------------------------------------------

	  deviceEnum.dpr
	  Voice Communicator II
	  Audio Tools - simple wave device enumeration application

	----------------------------------------------
	  This source code cannot be used without
	  proper permission granted to you as a private
	  person or an entity by the Lake of Soft, Ltd

	  Visit http://lakeofsoft.com/ for details.

	  Copyright (c) 2001, 2002 Lake of Soft, Ltd
		     All rights reserved
	----------------------------------------------

	  created by:
		Lake, 13 Mar 2002

	  modified by:
		Lake, 13 Mar 2002

	----------------------------------------------
*)

{$I unaDef.inc}

{$APPTYPE CONSOLE}

program
  deviceEnum;

uses
  Windows, unaTypes, unaUtils,
  MMSystem, unaMsAcmAPI, unaMsAcmClasses;

// --  --
procedure writeFormat(value: unsigned; const formatName: string);
begin
  if (0 <> value) then
    infoMessage('<dev_format>' + formatName + '</dev_format>');
end;

// --  --
procedure writeStdFormats(value: unsigned);
begin
  //
  infoMessage('<dev_supported_by_flag_formats>');
  writeFormat(WAVE_FORMAT_1M08 and value, '11.025 kHz, mono, 8-bit');
  writeFormat(WAVE_FORMAT_1M16 and value, '11.025 kHz, mono, 16-bit');
  writeFormat(WAVE_FORMAT_1S08 and value, '11.025 kHz, stereo, 8-bit');
  writeFormat(WAVE_FORMAT_1S16 and value, '11.025 kHz, stereo, 16-bit');
  //
  writeFormat(WAVE_FORMAT_2M08 and value, '22.05 kHz, mono, 8-bit');
  writeFormat(WAVE_FORMAT_2M16 and value, '22.05 kHz, mono, 8-bit');
  writeFormat(WAVE_FORMAT_2S08 and value, '22.05 kHz, stereo, 8-bit');
  writeFormat(WAVE_FORMAT_2S16 and value, '22.05 kHz, stereo, 8-bit');
  //
  writeFormat(WAVE_FORMAT_4M08 and value, '44.1 kHz, mono, 8-bit');
  writeFormat(WAVE_FORMAT_4M08 and value, '44.1 kHz, mono, 16-bit');
  writeFormat(WAVE_FORMAT_4S08 and value, '44.1 kHz, stereo, 8-bit');
  writeFormat(WAVE_FORMAT_4S08 and value, '44.1 kHz, stereo, 16-bit');
  infoMessage('</dev_supported_by_flag_formats>');
end;

// --  --
procedure writeOtherFormat(isOut: bool; id, rate: unsigned; bits: byte; stereo: bool);
var
  W: WAVEFORMATEX;
  r: unsigned;
  hout: HWAVEOUT;
  hin: HWAVEIN;
begin
  W.wFormatTag := WAVE_FORMAT_PCM;
  W.nChannels := choice(stereo, unsigned(2), 1);
  W.nSamplesPerSec := rate;
  W.wBitsPerSample := bits;
  W.nBlockAlign := W.nChannels * (W.wBitsPerSample shr 3);
  W.nAvgBytesPerSec := W.nBlockAlign * W.nSamplesPerSec;
  W.cbSize := 0;

  if (isOut) then
    r := waveOutOpen(@hout, id, @W, 0, 0, WAVE_FORMAT_QUERY)
  else
    r := waveInOpen(@hin, id, @W, 0, 0, WAVE_FORMAT_QUERY);

  if (mmNoError(r)) then
    infoMessage('<supported_format rate="' + int2Str(rate) + '" bits="' + int2Str(bits) + '" stereo="' + choice(stereo, '1', '0') + '" />')
end;

// --  --
procedure writeOtherFormats(isOut: bool; id: unsigned);
begin
  infoMessage('<dev_supported_by_device_formats>');
  //
  writeOtherFormat(isOut, id, 4000, 8, false);
  writeOtherFormat(isOut, id, 4000, 8, true);
  writeOtherFormat(isOut, id, 4000, 16, false);
  writeOtherFormat(isOut, id, 4000, 16, true);
  //
  writeOtherFormat(isOut, id, 8000, 8, false);
  writeOtherFormat(isOut, id, 8000, 8, true);
  writeOtherFormat(isOut, id, 8000, 16, false);
  writeOtherFormat(isOut, id, 8000, 16, true);
  //
  writeOtherFormat(isOut, id, 11025, 8, false);
  writeOtherFormat(isOut, id, 11025, 8, true);
  writeOtherFormat(isOut, id, 11025, 16, false);
  writeOtherFormat(isOut, id, 11025, 16, true);
  //
  writeOtherFormat(isOut, id, 12000, 8, false);
  writeOtherFormat(isOut, id, 12000, 8, true);
  writeOtherFormat(isOut, id, 12000, 16, false);
  writeOtherFormat(isOut, id, 12000, 16, true);
  //
  writeOtherFormat(isOut, id, 16000, 8, false);
  writeOtherFormat(isOut, id, 16000, 8, true);
  writeOtherFormat(isOut, id, 16000, 16, false);
  writeOtherFormat(isOut, id, 16000, 16, true);
  //
  writeOtherFormat(isOut, id, 22050, 8, false);
  writeOtherFormat(isOut, id, 22050, 8, true);
  writeOtherFormat(isOut, id, 22050, 16, false);
  writeOtherFormat(isOut, id, 22050, 16, true);
  //
  writeOtherFormat(isOut, id, 24000, 8, false);
  writeOtherFormat(isOut, id, 24000, 8, true);
  writeOtherFormat(isOut, id, 24000, 16, false);
  writeOtherFormat(isOut, id, 24000, 16, true);
  //
  writeOtherFormat(isOut, id, 32000, 8, false);
  writeOtherFormat(isOut, id, 32000, 8, true);
  writeOtherFormat(isOut, id, 32000, 16, false);
  writeOtherFormat(isOut, id, 32000, 16, true);
  //
  writeOtherFormat(isOut, id, 44100, 8, false);
  writeOtherFormat(isOut, id, 44100, 8, true);
  writeOtherFormat(isOut, id, 44100, 16, false);
  writeOtherFormat(isOut, id, 44100, 16, true);
  //
  writeOtherFormat(isOut, id, 48000, 8, false);
  writeOtherFormat(isOut, id, 48000, 8, true);
  writeOtherFormat(isOut, id, 48000, 16, false);
  writeOtherFormat(isOut, id, 48000, 16, true);
  //
  writeOtherFormat(isOut, id, 88200, 8, false);
  writeOtherFormat(isOut, id, 88200, 8, true);
  writeOtherFormat(isOut, id, 88200, 16, false);
  writeOtherFormat(isOut, id, 88200, 16, true);
  //
  writeOtherFormat(isOut, id, 96000, 8, false);
  writeOtherFormat(isOut, id, 96000, 8, true);
  writeOtherFormat(isOut, id, 96000, 16, false);
  writeOtherFormat(isOut, id, 96000, 16, true);
  //
  infoMessage('</dev_supported_by_device_formats>');
end;

// -- main --

var
  i: int;
  inCaps: WAVEINCAPS;
  outCaps: WAVEOUTCAPS;
begin
  infoMessage('<?xml version="1.0" encoding="UTF-8" standalone="yes"?>');
  infoMessage('<deviceEnum>');
  infoMessage('<info>' +
		'<author>deviceEnum, Version 1.0</author>' +
		'<copyright>Copyright (c) 2002 Lake of Soft, Ltd</copyright>' +
		'<legal>Freeware version, visit http://lakeofsoft.com/ for more information</legal>' +
	      '</info>');

  // input devices
  infoMessage('<input>');
  for i := 0 to unaWaveInDevice.getDeviceCount() - 1 do
    if (unaWaveInDevice.getCaps(i, inCaps)) then begin
      //
      infoMessage('<device id="' + int2Str(i) + '" mid="' + int2Str(inCaps.wMid) + '" pid="' + int2Str(inCaps.wPid) + '">');
      infoMessage('<dev_version>' + int2Str(inCaps.vDriverVersion shr 8) + '.' + int2Str(inCaps.vDriverVersion and $0F) + '</dev_version>');
      infoMessage('<dev_name>' + inCaps.szPname + '</dev_name>');
      writeStdFormats(inCaps.dwFormats);
      writeOtherFormats(false, i);
      infoMessage('<dev_support_channels mono="' + choice(0 <> (inCaps.wChannels and 1), '1', '0') + '" stereo="' + choice(0 <> (inCaps.wChannels and 2), '1', '0') + '" />');
      infoMessage('</device>');
    end;
  //
  infoMessage('</input>');

  infoMessage('<output>');
  for i := 0 to unaWaveOutDevice.getDeviceCount() - 1 do
    if (unaWaveOutDevice.getCaps(i, outCaps)) then begin
      //
      infoMessage('<device id="' + int2Str(i) + '" mid="' + int2Str(outCaps.wMid) + '" pid="' + int2Str(outCaps.wPid) + '">');
      infoMessage('<dev_version>' + int2Str(outCaps.vDriverVersion shr 8) + '.' + int2Str(outCaps.vDriverVersion and $0F) + '</dev_version>');
      infoMessage('<dev_name>' + outCaps.szPname + '</dev_name>');
      writeStdFormats(outCaps.dwFormats);
      writeOtherFormats(true, i);
      infoMessage('<dev_support_channels mono="' + choice(0 <> (outCaps.wChannels and 1), '1', '0') + '" stereo="' + choice(0 <> (outCaps.wChannels and 2), '1', '0') + '" />');
      infoMessage('<dev_support_features>' +
		  choice(0 <> (WAVECAPS_LRVOLUME and outCaps.dwSupport), '"separate left and right volume control" ', ' ') +
		  choice(0 <> (WAVECAPS_PITCH and outCaps.dwSupport), '"pitch control" ', ' ') +
		  choice(0 <> (WAVECAPS_PLAYBACKRATE and outCaps.dwSupport), '"playback rate control" ', ' ') +
		  choice(0 <> (WAVECAPS_SYNC and outCaps.dwSupport), '"driver is synchronous and will block while playing a buffer" ', ' ') +
		  choice(0 <> (WAVECAPS_VOLUME and outCaps.dwSupport), '"volume control" ', ' ') +
		  choice(0 <> (WAVECAPS_SAMPLEACCURATE and outCaps.dwSupport), '"returns sample-accurate position information" ', ' ') +
		  '</dev_support_features>');
      infoMessage('</device>');
    end;
  //
  infoMessage('</output>');

  infoMessage('</deviceEnum>');
end.

